import { forwardRef, ChangeDetectionStrategy, Component, EventEmitter, Input, KeyValueDiffers, NgZone, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

function normalizeLineEndings(str) {
    if (!str) {
        return str;
    }
    return str.replace(/\r\n|\r/g, '\n');
}
var CodemirrorComponent = /** @class */ (function () {
    function CodemirrorComponent(_differs, _ngZone) {
        this._differs = _differs;
        this._ngZone = _ngZone;
        this.className = '';
        this.name = 'codemirror';
        this.autoFocus = false;
        this.preserveScrollPosition = false;
        this.cursorActivity = new EventEmitter();
        this.focusChange = new EventEmitter();
        this.scroll = new EventEmitter();
        this.value = '';
        this.disabled = false;
        this.isFocused = false;
        this.onChange = function (_) { };
        this.onTouched = function () { };
    }
    Object.defineProperty(CodemirrorComponent.prototype, "options", {
        set: function (value) {
            this._options = value;
            if (!this._differ && value) {
                this._differ = this._differs.find(value).create();
            }
        },
        enumerable: true,
        configurable: true
    });
    CodemirrorComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (!this.ref) {
            return;
        }
        var fromTextArea = require('codemirror').fromTextArea;
        this._ngZone.runOutsideAngular(function () {
            _this.codeMirror = fromTextArea(_this.ref.nativeElement, _this._options);
            _this.codeMirror.on('cursorActivity', _this.cursorActive.bind(_this));
            _this.codeMirror.on('scroll', _this.scrollChanged.bind(_this));
            _this.codeMirror.on('blur', function () { return _this._ngZone.run(function () { return _this.focusChanged(false); }); });
            _this.codeMirror.on('focus', function () { return _this._ngZone.run(function () { return _this.focusChanged(true); }); });
            _this.codeMirror.on('change', function (cm, change) { return _this._ngZone.run(function () { return _this.codemirrorValueChanged(cm, change); }); });
            _this.codeMirror.setValue(_this.value);
        });
    };
    CodemirrorComponent.prototype.ngDoCheck = function () {
        var _this = this;
        if (!this._differ) {
            return;
        }
        var changes = this._differ.diff(this._options);
        if (changes) {
            changes.forEachChangedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
            changes.forEachAddedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
            changes.forEachRemovedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
        }
    };
    CodemirrorComponent.prototype.ngOnDestroy = function () {
        if (this.codeMirror) {
            this.codeMirror.toTextArea();
        }
    };
    CodemirrorComponent.prototype.codemirrorValueChanged = function (cm, change) {
        if (change.origin !== 'setValue') {
            this.value = cm.getValue();
            this.onChange(this.value);
        }
    };
    CodemirrorComponent.prototype.setOptionIfChanged = function (optionName, newValue) {
        if (!this.codeMirror) {
            return;
        }
        this.codeMirror.setOption(optionName, newValue);
    };
    CodemirrorComponent.prototype.focusChanged = function (focused) {
        this.onTouched();
        this.isFocused = focused;
        this.focusChange.emit(focused);
    };
    CodemirrorComponent.prototype.scrollChanged = function (cm) {
        this.scroll.emit(cm.getScrollInfo());
    };
    CodemirrorComponent.prototype.cursorActive = function (cm) {
        this.cursorActivity.emit(cm);
    };
    CodemirrorComponent.prototype.writeValue = function (value) {
        if (value === null || value === undefined) {
            return;
        }
        if (!this.codeMirror) {
            this.value = value;
            return;
        }
        var cur = this.codeMirror.getValue();
        if (value !== cur &&
            normalizeLineEndings(cur) !== normalizeLineEndings(value)) {
            this.value = value;
            if (this.preserveScrollPosition) {
                var prevScrollPosition = this.codeMirror.getScrollInfo();
                this.codeMirror.setValue(this.value);
                this.codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
            }
            else {
                this.codeMirror.setValue(this.value);
            }
        }
    };
    CodemirrorComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    CodemirrorComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    CodemirrorComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.setOptionIfChanged('readOnly', this.disabled);
    };
    return CodemirrorComponent;
}());
CodemirrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-codemirror',
                template: "\n  <textarea\n    [name]=\"name\"\n    class=\"ngx-codemirror {{ className }}\"\n    [class.ngx-codemirror--focused]=\"isFocused\"\n    autocomplete=\"off\"\n    [autofocus]=\"autoFocus\"\n    #ref>\n  </textarea>\n  ",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(function () { return CodemirrorComponent; }),
                        multi: true,
                    },
                ],
                preserveWhitespaces: false,
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
CodemirrorComponent.ctorParameters = function () { return [
    { type: KeyValueDiffers, },
    { type: NgZone, },
]; };
CodemirrorComponent.propDecorators = {
    "className": [{ type: Input },],
    "name": [{ type: Input },],
    "autoFocus": [{ type: Input },],
    "options": [{ type: Input },],
    "preserveScrollPosition": [{ type: Input },],
    "cursorActivity": [{ type: Output },],
    "focusChange": [{ type: Output },],
    "scroll": [{ type: Output },],
    "ref": [{ type: ViewChild, args: ['ref',] },],
};
var CodemirrorModule = /** @class */ (function () {
    function CodemirrorModule() {
    }
    return CodemirrorModule;
}());
CodemirrorModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [CodemirrorComponent],
                declarations: [CodemirrorComponent],
                providers: [],
            },] },
];
CodemirrorModule.ctorParameters = function () { return []; };

export { CodemirrorComponent, CodemirrorModule };
//# sourceMappingURL=ctrl-ngx-codemirror.js.map
